@@ -1,4 +1,4 @@
-""" support for providing temporary directories to test functions.  """
+"""Support for providing temporary directories to test functions."""
 import os
 import re
 import tempfile
@@ -13,21 +13,25 @@ from .pathlib import LOCK_TIMEOUT
 from .pathlib import make_numbered_dir
 from .pathlib import make_numbered_dir_with_cleanup
 from .pathlib import Path
+from _pytest.compat import final
+from _pytest.config import Config
 from _pytest.fixtures import FixtureRequest
 from _pytest.monkeypatch import MonkeyPatch
 
 
+@final
 @attr.s
 class TempPathFactory:
     """Factory for temporary directories under the common base temp directory.
 
-    The base directory can be configured using the ``--basetemp`` option."""
+    The base directory can be configured using the ``--basetemp`` option.
+    """
 
     _given_basetemp = attr.ib(
-        type=Path,
-        # using os.path.abspath() to get absolute path instead of resolve() as it
-        # does not work the same in all platforms (see #4427)
-        # Path.absolute() exists, but it is not public (see https://bugs.python.org/issue25012)
+        type=Optional[Path],
+        # Use os.path.abspath() to get absolute path instead of resolve() as it
+        # does not work the same in all platforms (see #4427).
+        # Path.absolute() exists, but it is not public (see https://bugs.python.org/issue25012).
         # Ignore type because of https://github.com/python/mypy/issues/6172.
         converter=attr.converters.optional(
             lambda p: Path(os.path.abspath(str(p)))  # type: ignore
@@ -37,15 +41,13 @@ class TempPathFactory:
     _basetemp = attr.ib(type=Optional[Path], default=None)
 
     @classmethod
-    def from_config(cls, config) -> "TempPathFactory":
-        """
-        :param config: a pytest configuration
-        """
+    def from_config(cls, config: Config) -> "TempPathFactory":
+        """Create a factory according to pytest configuration."""
         return cls(
             given_basetemp=config.option.basetemp, trace=config.trace.get("tmpdir")
         )
 
-    def _ensure_relative_to_basetemp(self, basename: str):
+    def _ensure_relative_to_basetemp(self, basename: str) -> str:
         basename = os.path.normpath(basename)
         if (self.getbasetemp() / basename).resolve().parent != self.getbasetemp():
             raise ValueError(
@@ -54,18 +56,18 @@ class TempPathFactory:
         return basename
 
     def mktemp(self, basename: str, numbered: bool = True) -> Path:
-        """Creates a new temporary directory managed by the factory.
+        """Create a new temporary directory managed by the factory.
 
         :param basename:
             Directory base name, must be a relative path.
 
         :param numbered:
-            If True, ensure the directory is unique by adding a number
-            prefix greater than any existing one: ``basename="foo"`` and ``numbered=True``
+            If ``True``, ensure the directory is unique by adding a numbered
+            suffix greater than any existing one: ``basename="foo-"`` and ``numbered=True``
             means that this function will create directories named ``"foo-0"``,
             ``"foo-1"``, ``"foo-2"`` and so on.
 
-        :return:
+        :returns:
             The path to the new directory.
         """
         basename = self._ensure_relative_to_basetemp(basename)
@@ -78,7 +80,7 @@ class TempPathFactory:
         return p
 
     def getbasetemp(self) -> Path:
-        """ return base temporary directory. """
+        """Return base temporary directory."""
         if self._basetemp is not None:
             return self._basetemp
 
@@ -103,30 +105,26 @@ class TempPathFactory:
         return t
 
 
+@final
 @attr.s
 class TempdirFactory:
-    """
-    backward comptibility wrapper that implements
-    :class:``py.path.local`` for :class:``TempPathFactory``
-    """
+    """Backward comptibility wrapper that implements :class:``py.path.local``
+    for :class:``TempPathFactory``."""
 
     _tmppath_factory = attr.ib(type=TempPathFactory)
 
     def mktemp(self, basename: str, numbered: bool = True) -> py.path.local:
-        """
-        Same as :meth:`TempPathFactory.mkdir`, but returns a ``py.path.local`` object.
-        """
+        """Same as :meth:`TempPathFactory.mktemp`, but returns a ``py.path.local`` object."""
         return py.path.local(self._tmppath_factory.mktemp(basename, numbered).resolve())
 
-    def getbasetemp(self):
-        """backward compat wrapper for ``_tmppath_factory.getbasetemp``"""
+    def getbasetemp(self) -> py.path.local:
+        """Backward compat wrapper for ``_tmppath_factory.getbasetemp``."""
         return py.path.local(self._tmppath_factory.getbasetemp().resolve())
 
 
 def get_user() -> Optional[str]:
     """Return the current user name, or None if getuser() does not work
-    in the current environment (see #1010).
-    """
+    in the current environment (see #1010)."""
     import getpass
 
     try:
@@ -135,7 +133,7 @@ def get_user() -> Optional[str]:
         return None
 
 
-def pytest_configure(config) -> None:
+def pytest_configure(config: Config) -> None:
     """Create a TempdirFactory and attach it to the config object.
 
     This is to comply with existing plugins which expect the handler to be
@@ -152,16 +150,14 @@ def pytest_configure(config) -> None:
 
 @pytest.fixture(scope="session")
 def tmpdir_factory(request: FixtureRequest) -> TempdirFactory:
-    """Return a :class:`_pytest.tmpdir.TempdirFactory` instance for the test session.
-    """
+    """Return a :class:`_pytest.tmpdir.TempdirFactory` instance for the test session."""
     # Set dynamically by pytest_configure() above.
     return request.config._tmpdirhandler  # type: ignore
 
 
 @pytest.fixture(scope="session")
 def tmp_path_factory(request: FixtureRequest) -> TempPathFactory:
-    """Return a :class:`_pytest.tmpdir.TempPathFactory` instance for the test session.
-    """
+    """Return a :class:`_pytest.tmpdir.TempPathFactory` instance for the test session."""
     # Set dynamically by pytest_configure() above.
     return request.config._tmp_path_factory  # type: ignore
 
@@ -175,12 +171,12 @@ def _mk_tmp(request: FixtureRequest, factory: TempPathFactory) -> Path:
 
 
 @pytest.fixture
-def tmpdir(tmp_path):
-    """Return a temporary directory path object
-    which is unique to each test function invocation,
-    created as a sub directory of the base temporary
-    directory.  The returned object is a `py.path.local`_
-    path object.
+def tmpdir(tmp_path: Path) -> py.path.local:
+    """Return a temporary directory path object which is unique to each test
+    function invocation, created as a sub directory of the base temporary
+    directory.
+
+    The returned object is a `py.path.local`_ path object.
 
     .. _`py.path.local`: https://py.readthedocs.io/en/latest/path.html
     """
@@ -189,15 +185,15 @@ def tmpdir(tmp_path):
 
 @pytest.fixture
 def tmp_path(request: FixtureRequest, tmp_path_factory: TempPathFactory) -> Path:
-    """Return a temporary directory path object
-    which is unique to each test function invocation,
-    created as a sub directory of the base temporary
-    directory.  The returned object is a :class:`pathlib.Path`
-    object.
+    """Return a temporary directory path object which is unique to each test
+    function invocation, created as a sub directory of the base temporary
+    directory.
+
+    The returned object is a :class:`pathlib.Path` object.
 
     .. note::
 
-        in python < 3.6 this is a pathlib2.Path
+        In python < 3.6 this is a pathlib2.Path.
     """
 
     return _mk_tmp(request, tmp_path_factory)
